################################################################################
#
# Table classes
#
################################################################################

itcl::class table_class {
    
    variable table_name
    
    constructor {name} {
        set table_name $name ;# the sql table name
    }
    
    public method get_table_name {} {
        return $table_name
    }
    
    public method get_conditions {} 
    
    public method conditions_available {} {
        return 0;
    }
    
    # returns true if this table is currently involved in the query
    public method involved {} {
        global report_level
        # General: a table is involved if: (a) there are conditions existing for this table or
        # (b) if it is the current main table
        
        if {[conditions_available] || [get_table_name_of_level $report_level] eq $table_name} {
            return 1
        } else  {
            return 0
        }
    }
    
}

body table_class::get_conditions {} {
    return ""
}

itcl::class campaign_class {
    inherit table_class
    
    constructor {name} {
        table_class::constructor $name
    } {}
    
    public method get_conditions {} {
        global campaign_wildcards
        
        set cond ""
        
        if {[$this conditions_available]} {
            if {[.campaign_combo getvalue] != 0} {
                # individual campaign is selected
                # get current campaign name
                set name [.campaign_combo get]
                append cond "name = '" $name "' "
            } else  {
                if {$campaign_wildcards ne ""} {
                    # wildcards have been entered by user
                    append cond "name like '" $campaign_wildcards "' "
                }
            }
        }
        
        return $cond
    }
    
    public method conditions_available {} {
        global campaign_wildcards
        
        set i [.campaign_combo getvalue]
        
        if {$i != 0 || $campaign_wildcards ne ""} {
            return 1
        } else  {
            return 0
        }
    }
    
}

itcl::class species_class {
    inherit table_class
    
    constructor {name} {
        table_class::constructor $name
    } {}
    
    public method get_conditions {} {
        global species_wildcards
        set cond ""
        
        if {[conditions_available]} {
            
            if {[.species_combo getvalue] != 0} {
                # individual species is selected
                set cond [str_concat "folder_name = '" [.species_combo get] "'"]
            } else  {
                if {$species_wildcards ne ""} {
                    # wildcards have been entered by user
                    set cond [str_concat "folder_name like '" $species_wildcards "'"]
                }
            }
        }
        return $cond
        
    }
    
    public method conditions_available {} {
        global species_wildcards
        #	set i $species_combo_cur_sel
        set i [.species_combo getvalue]
        
        if {$i != 0 || $species_wildcards ne ""} {
            return 1
        } else  {
            return 0
        }
    }
    
}

itcl::class site_class {
    inherit table_class
    
    constructor {name} {
        table_class::constructor $name
    } {}
    
    public method get_conditions {} {
        set cond ""
        
        if {[conditions_available]} {
            set f_date [.from_date cget -text]
            set t_date [.to_date cget -text]
            set min_alt [.min_alt get]
            set max_alt [.max_alt get]
            
            if {$f_date ne ""} {conc_cond cond [str_concat "capture_date >= '" $f_date "'"]}
            if {$t_date ne ""} {conc_cond cond [str_concat "capture_date <= '" $t_date "'"]}
            if {$min_alt ne ""} {conc_cond cond [str_concat "site.altitude >= " $min_alt]}
            if {$max_alt ne ""} {conc_cond cond [str_concat "site.altitude <= " $max_alt]}
            
        }
        
        return $cond
        
    }
    
    public method conditions_available {} {
        # site conds are existing if at least one of the date fields is not empty
        # or if one of the altitude fields is filled in
        set f_date [.from_date cget -text]
        set t_date [.to_date cget -text]
        set min_alt [.min_alt get]
        set max_alt [.min_alt get]
        
        if {[str_concat $f_date $t_date $min_alt $max_alt] ne ""} {
            return 1
        }
        return 0
    }
    
}

itcl::class spectrum_class {
    inherit table_class
    
    constructor {name} {
        table_class::constructor $name
    } {}
}
