################################################################################
#
# GUI Creation
# ------------
#
################################################################################

#--------------------------------------------------
# Window Events Handler Registration
#--------------------------------------------------
wm protocol . WM_DELETE_WINDOW [list closeMainWindow .]


proc create_GUI {} {
    global startup db_handle debug_txt_setting
    
    
    
    # Get study (campaign) names and build list of strings
    
    foreach res [mysqlsel $db_handle  {select NAME FROM study} -flatlist] {
        append campaigns " \"" $res "\" "
    }
    
    
    # Header
    frame .header_frm
    
    # Title
    label .title -text "SpectraProc Query" -anchor "w" -font "Arial 18 bold"
    label .subtitle -text "A TCL/TK Interface" -anchor "w" -font "Arial 14"
    grid .title -in .header_frm -row 1 -column 1 -sticky nw
    grid .subtitle -in .header_frm -row 2 -column 1 -sticky sw
    
    
    if {$debug_txt_setting == 1} {
        text .debug_txt -width 50 -height 10
        grid .debug_txt -in .header_frm -row 1 -column 2 -sticky sw -rowspan 2
    }
    
    pack .header_frm -anchor w
    
    
    # Main
    frame .main
    
    # SQL Query
    labelframe .sql_query_frm -borderwidth 1 -relief sunken -padx 2 -pady 2 -text "SQL Query (autobuilt)"
    text .sql_query_txt -width 50 -height 6
    grid .sql_query_txt -in .sql_query_frm
    grid .sql_query_frm -in .main -row 2 -column 2 -rowspan 2
    
    # Resulting Rows (Row count)
    labelframe .row_cnt_frm -borderwidth 1 -relief sunken -padx 2 -pady 2 -text "Result Set Information"
    label .row_cnt_lab -text "Number of resulting rows by current query: "
    label .row_cnt_number -width 10 -bd 1 -relief sunken
    grid .row_cnt_lab -in .row_cnt_frm -row 1 -column 1
    grid .row_cnt_number -in .row_cnt_frm -row 1 -column 2
    grid .row_cnt_frm -in .main -row 4 -column 2
    
    
    # Campaign
    labelframe .camp_frm -borderwidth 1 -relief sunken -padx 2 -pady 2 -text "Campaign"
    ComboBox .campaign_combo -borderwidth 1 -editable false 
    # add entries
    set campaign_listbox [.campaign_combo getlistbox]
    $campaign_listbox insert end "All"
    foreach elem $campaigns {
        $campaign_listbox insert end $elem
    }
    .campaign_combo setvalue first
    
    entry .camp_wildcards
    bind .camp_wildcards "<KeyRelease>" camp_wildcards_changed
    label .camp_wildcards_lab -text "Use wildcards: '%' for multiple chars and '_' for single chars"
    grid .campaign_combo -in .camp_frm -row 1 -column 1 -sticky sw
    grid .camp_wildcards -in .camp_frm -row 1 -column 2 -sticky sw
    grid .camp_wildcards_lab -in .camp_frm -row 2 -column 1 -sticky sw -columnspan 2
    grid .camp_frm -in .main -row 1 -column 1 -sticky ew
    
    
    
    # Species
    labelframe .species_frm -borderwidth 1 -relief sunken -padx 2 -pady 2 -text "Species"
    label .folder_lab -text "Folder name: "
    ComboBox .species_combo -borderwidth 1 -editable false
    fill_species_combo
    
    entry .folder_wildcards
    bind .folder_wildcards "<KeyRelease>" spec_wildcards_changed
    grid .folder_lab -in .species_frm -row 1 -column 1 -sticky sw
    grid .species_combo -in .species_frm -row 1 -column 2 -sticky sw
    grid .folder_wildcards -in .species_frm -row 1 -column 3 -sticky sw
    grid .species_frm -in .main -row 2 -column 1 -sticky ew
    
    # Collection Dates
    labelframe .dates_frm -borderwidth 1 -relief sunken -padx 2 -pady 2 -text "Collection Dates"
    label .date_from -text "From: "
    entry .from_DD -width 2
    bind .from_DD "<KeyRelease>" date_changed
    entry .from_MM -width 2
    bind .from_MM "<KeyRelease>" date_changed
    entry .from_YYYY -width 4
    bind .from_YYYY "<KeyRelease>" date_changed
    label .from_date -width 10 -bd 1 -relief sunken ;# display date build result
    label .date_to -text "To: "
    entry .to_DD -width 2
    bind .to_DD "<KeyRelease>" date_changed
    entry .to_MM -width 2
    bind .to_MM "<KeyRelease>" date_changed
    entry .to_YYYY -width 4
    bind .to_YYYY "<KeyRelease>" date_changed
    label .to_date -width 10 -bd 1 -relief sunken ;# display date build result
    grid .date_from -in .dates_frm -row 1 -column 1 -sticky sw
    grid .from_DD -in .dates_frm -row 1 -column 2 -sticky sw
    grid .from_MM -in .dates_frm -row 1 -column 3 -sticky sw
    grid .from_YYYY -in .dates_frm -row 1 -column 4 -sticky sw
    grid .from_date -in .dates_frm -row 1 -column 5 -sticky se  -padx 5
    grid .date_to -in .dates_frm -row 2 -column 1 -sticky sw
    grid .to_DD -in .dates_frm -row 2 -column 2 -sticky sw
    grid .to_MM -in .dates_frm -row 2 -column 3 -sticky sw
    grid .to_YYYY -in .dates_frm -row 2 -column 4 -sticky sw
    grid .to_date -in .dates_frm -row 2 -column 5 -sticky sw -padx 5
    grid .dates_frm -in .main -row 3 -column 1 -sticky ew
    
    
    # Spatial Position
    labelframe .spatial_frm -borderwidth 1 -relief sunken -padx 2 -pady 2 -text "Spatial Attributes"
    label .spatial_lab -text "Altitude\[m\]: "
    entry .min_alt -width 5
    bind .min_alt "<KeyRelease>" altitude_changed
    label .inbetween_lab -text "<->"
    entry .max_alt -width 5
    bind .max_alt "<KeyRelease>" altitude_changed
    grid .spatial_lab -in .spatial_frm -row 1 -column 1
    grid .min_alt -in .spatial_frm -row 1 -column 2
    grid .inbetween_lab -in .spatial_frm -row 1 -column 3
    grid .max_alt -in .spatial_frm -row 1 -column 4
    grid .spatial_frm -in .main -row 4 -column 1
    
    
    # Report Level
    labelframe .rep_level_frm -borderwidth 1 -relief sunken -padx 2 -pady 2 -text "Report Level"
    radiobutton .campaign_level -text "Campaign" -variable report_level -value 1 -command report_level_changed
    radiobutton .species_level -text "Species" -variable report_level -value 2 -command report_level_changed
    radiobutton .site_level -text "Site" -variable report_level -value 3 -command report_level_changed
    radiobutton .spectrum_level -text "Spectrum" -variable report_level -value 4 -command report_level_changed
    grid .campaign_level -in .rep_level_frm -row 1 -column 1 -sticky w
    grid .species_level -in .rep_level_frm -row 1 -column 2 -sticky w
    grid .site_level -in .rep_level_frm -row 2 -column 1 -sticky w
    grid .spectrum_level -in .rep_level_frm -row 2 -column 2 -sticky w
    grid .rep_level_frm -in .main -row 1 -column 2
    
    # Action Buttons
    frame .action_button_frm -padx 2 -pady 2
    button .exec_query -text "Exec Query" -command "exec_query"
    button .reset -text "Reset" -command "reset"
    grid .exec_query -in .action_button_frm  -row 1 -column 1 -padx 2 -pady 2
    grid .reset -in .action_button_frm  -row 1 -column 2 -padx 2 -pady 2
    grid .action_button_frm -in .main -row 5 -column 2 -padx 2 -pady 2
    
    
    
    grid configure .camp_frm .species_frm .dates_frm .spatial_frm\
            .rep_level_frm .sql_query_frm .row_cnt_frm -sticky ew -padx 20 -pady 10
    grid configure .main -sticky w
    
    pack .main -anchor w
    
    
    set startup 0
    
    
    # set commands here (because they would try to call non existent widgets if called further up)
    .campaign_combo configure -modifycmd campaign_combo_changed
    .species_combo configure -modifycmd species_combo_changed
    
    
    
}