
#--------------------------------------------------
# Globals
#--------------------------------------------------

set window_number 0
set ids "" ;# this holds a list of the primary key id's


	
# build the query used for the reports, based on the id's supplied by the query
# built in the main window
proc build_report_query {report_level} {
    global sql_select_query db_handle ids
    
    set ids ""
    set join_str ""
	
    set sel_attributes [get_attributes $report_level as_sql_string 1]
    set table [get_table_name_of_level $report_level]
    set conds [get_primary_key_column_name $table]
    
    # create list of id's from original select query
    set ids [mysqlsel $db_handle	$sql_select_query -flatlist] 
        
    append conds [str_concat " in (" [join $ids ", "] ")"]
	
	set report_query [assemble_sql_query $sel_attributes $table $conds]
	
	return $report_query	
}

# return attributes either as as_sql_string or as_list
proc get_attributes {report_level {return_type as_sql_string} {prefixing 0}} {
    
    set tablename [get_table_name_of_level $report_level]
    
	switch $report_level {
		1 { ;# "Campaign"
            set sel_attributes {name description datapath}
		}
		2 { ;# "Species"
			set sel_attributes {folder_name common_name latin_name}
		}
		3 { ;# "Site"
			set sel_attributes {capture_date longitude latitude altitude }
		}
		4 { ;# "Spectrum"
			set sel_attributes {reflectances disabled spectrum_no pathname asd_comment}
		}
    }
    
    if {$prefixing == 1} {
        set sel_attributes [prefix_with_tablename $tablename $sel_attributes]
    }
	
	switch $return_type {
		as_sql_string {
			return [join $sel_attributes ", "]
		}
		as_list {
			return $sel_attributes
		}
	}	
}

	
proc create_report_window_name {} {
    global window_number
    # build window name
    return [str_concat ".rep_win_" [incr window_number]]
    
}

proc get_level_name {level} {
    switch $level {
        1 { ;# "Campaign"
            return "Campaign"
        }
        2 { ;# "Species"
            return "Species"
        }
        3 { ;# "Site"
            return "Site"
        }
        4 { ;# "Spectrum"
            return "Spectrum"
        }
    }
}

# returns a list that contains: the data type, the width of the column and cell type for each attribute
# cell types are: normal, text, graph
# one of the concepts is: minimial width should at least be same as the length of the column title, i.e.
# attribute name (column name)
proc get_formatting_list {level} {
    set attr [get_attributes $level as_list]
    
    # get data type and the maximal length of the varchar columns
    set type_and_max_col_length [get_datatype_and_varchar_length [get_table_name_of_level $level] $attr]
    
    foreach at $attr {t l} $type_and_max_col_length {
        lappend format_list $t
        
        if {$t eq "blob"} {
            set width 50
            set type graph
        }
        if {$l < 50 && $t eq "varchar"} {
            set width [max 20 [llength $at]]
            set type normal            
        }
        if {$l >= 50 && $t eq "varchar"} {
            set width [max 30 [llength $at]]
            set type text            
        }
        if {$t eq "int"} {
            set width [max 10 [string length $at]]
            set type normal
        }
        if {$t eq "datetime"} {
            set width [max 20 [string length $at]]
            set type normal
        }
        if {$t eq "tinyint"} {
            set width [max 10 [string length $at]]
            set type checkbox
        }
        
        
        lappend format_list $width $type
    }
    
    return $format_list
    
}

# toggles the disabled flag of the spectrum
proc alter_spectrum_disabled_state {spectrum_id} {
    global db_handle
    
    # get current state in database
    set query "select disabled from spectrum where spectrum_id = $spectrum_id"
    set state [mysqlsel $db_handle $query -flatlist]            
    
    if {$state == 0} {
        set new_state 1
    } else {
        set new_state 0
    }
    
    set query "update spectrum set disabled=$new_state where spectrum_id = $spectrum_id"
    mysqlexec $db_handle $query
    
}

proc create_query_result {level} {
    global window_number db_handle sql_count_query ids
    global SPECTRUM
    
    set building_time [time {
    
    set blobs_exist 0

    # Construct title
    set title [str_concat [get_level_name $level] " Query Results"]
    
    # create new toplevel window
    set rep_win [create_report_window_name]
    toplevel $rep_win
    wm title $rep_win [str_concat [get_level_name $level] " Query Results " $window_number]
    
    # create title label and pack it
    label $rep_win.title -text $title -font "Arial 14"
    label $rep_win.load_info -font "Arial 8"
    pack $rep_win.title
    pack $rep_win.load_info
    
    # create frame to hold the data table plus the scrollbars
    frame $rep_win.data_frm
    
    # get number of rows in order to create table big enough
    #set no_of_rows [mysqlsel $db_handle $sql_count_query -flatlist]
    # get it from label in main gui -> should be much faster!
    set no_of_rows [.row_cnt_number cget -text]
    incr no_of_rows ;# plus header row
    
    # get number of attributes
    set no_of_attr [llength [get_attributes $level as_list]]
    
    # create data table and scrollbars
    set dt [table $rep_win.data_frm.data_table -bd 1 -cols $no_of_attr -height 0 \
            -variable table_data$window_number -colwidth 30 -colstretchmode unset \
            -rowstretchmode unset -rowheight 2 -rows $no_of_rows\
            -yscrollcommand "$rep_win.data_frm.srl_y set" \
            -xscrollcommand "$rep_win.data_frm.srl_x set" -padx 4 -pady 4]
            
    scrollbar $rep_win.data_frm.srl_y -command "$dt yview" -orient v
    scrollbar $rep_win.data_frm.srl_x -command "$dt xview" -orient h
    
    # grid and pack
    grid $dt -row 1 -column 1 -sticky ewns
    grid $rep_win.data_frm.srl_y -row 1 -column 2 -sticky ns
    grid $rep_win.data_frm.srl_x -row 2 -column 1 -sticky ew
    grid rowconfigure $rep_win.data_frm 1 -weight 1
    grid columnconfigure $rep_win.data_frm 1 -weight 1
    pack $rep_win.data_frm -expand 1 -fill both
    
    # fill header row and tag it
    $dt set row 0,0 [get_attributes $level as_list]
    $dt tag row header 0
    $dt height 0 1
        
    set formatting_list [get_formatting_list $level]    
    
    # build the report query
    set report_query [build_report_query $level]
    
    # create list of species names if the current level is 'spectrum
    # the names are used for the title of the spectral plots
    # We just hope that the records are always returned in the same order
    # (given the id list for species and spectrum queries is the same)
    # Need to do this one by one ..
    if {$level == $SPECTRUM} {
        foreach id $ids {
            set name_query "select folder_name from species where species_id in (select species_id from spectrum where spectrum_id = $id)"
            lappend species_names [mysqlsel $db_handle	$name_query -flatlist]            
        }
    }
        
    # get the data
    mysqlsel $db_handle	$report_query
    set row 1
    set col 0
    set cnt 0
    while {[llength [set curr_row [mysql::fetch $db_handle]]]>0} {
        
        #.debug_txt insert end $curr_row
        
        foreach column $curr_row {data_type width cell_type} $formatting_list {
            
            if {$cell_type eq "normal"} {
                # put the column value into the normal cell
                $dt activate $row,$col
                $dt insert active end $column
            }
            
            if {$cell_type eq "text"} {

                # put the column value into a scrollable text widget
                
                # create a frame to act as embedded window
                set embd_frame [frame $dt.embd_frm$row$col]
                # create text and associated scrollbar
                text $embd_frame.text -width 25 -height 4 -relief flat -yscrollcommand "$embd_frame.srl_y set"
                scrollbar $embd_frame.srl_y -command "$embd_frame.text yview" -orient v
                
                # put column value into text
                $embd_frame.text insert end $column
                
                # grid and pack the embedded window
                grid $embd_frame.text -row 1 -column 1 -sticky ewns
                grid $embd_frame.srl_y  -row 1 -column 2 -sticky ns
                grid rowconfigure $embd_frame 1 -weight 1
                grid columnconfigure $embd_frame 1 -weight 1
                
                # embed the frame
                $dt window configure $row,$col -window $embd_frame -sticky ewns
            }
            
            if {$cell_type eq "graph"} {
                set blobs_exist 1
                # convert blob to floats
                binary scan $column f* refl

                set channel 350 ;# asd start channel
                set delta 10 ;# effects a crude downsampling of the data
                set i 0
                set reflectances {}
                
                # build a new list with x y values
                for {set c $channel} {$c < 2500} {incr c $delta} {
                    lappend reflectances $c [lindex $refl $i]
                    incr i $delta
                }
 
                # create a canvas and embed it
                canvas $dt.embd_canvas$row$col -background white -width 300 -height 200
                $dt window configure $row,$col -window $dt.embd_canvas$row$col -sticky ewns
                
                #
                # Create the plot with its x- and y-axes
                #
                set s [::Plotchart::createXYPlot $dt.embd_canvas$row$col {350.0 2500.0 550.0} {0.0 1.0 0.2}]
                
                foreach {x y} $reflectances {
                    $s plot series1 $x $y
                }
                
                
                
                
                $s title [lindex $species_names $cnt]
                $s xtext "Wavelength \[nm\]"
                $s ytext "Refl."
                
            
            }
            
            if {$cell_type eq "checkbox"} {                
                # create a checkbox and embed it
                checkbutton $dt.chk$row$col -command "alter_spectrum_disabled_state [lindex $ids $cnt]"
                $dt window configure $row,$col -window $dt.chk$row$col -sticky ewns

		    if {$column == 0} {
                    $dt.chk$row$col deselect
                   } else {
                    $dt.chk$row$col select
                   }

            }
            
            
            incr col            
        }
        # tag the row
        $dt tag row content $row
        incr row
        set col 0
        incr cnt
    }
    
    
    # configure the tags (for nicer display, yeah, right ...)
    $dt tag configure header -bg grey90 -fg black -justify left -anchor w -relief solid -wrap 1    
    $dt tag configure content -bg white -fg black -justify left -anchor nw -relief solid -wrap 1
    
    # clear the selection 
    #$dt selection clear all
    
    # activte non existing row to desactivate the last used cell
    $dt activate $row,0
    
    # no changes in the table by user
    $dt configure -state disabled
    
    # set the width of all columns
    set col 0
    foreach {data_type width cell_type} $formatting_list {
        $dt width $col $width
        incr col
    }
    
    # if there were blobs reconfigure the row height for all columns
    # and widen the reflectance column
    if {$blobs_exist} {
        $dt configure -rowheight 13        
    }
    
    }] ;# time end
    
    # set the loading information
    set seconds [expr [lindex $building_time 0] / (1000 * 1000)]
    set milliseconds [expr [lindex $building_time 0] / 1000]
    $rep_win.load_info configure -text [str_concat [.row_cnt_number cget -text] " rows loaded in " $seconds "." $milliseconds " seconds"]
        
    
}
	



